﻿using Eingabeverwaltung.Metamodell;
using Eingabeverwaltung.Parser;
using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace Eingabeverwaltung
{
    public class SimpleEingabeverwalter : AbstractEingabeverwalter
    {
        private string adtPath;
        private string taskPath;
        private string proofPath;
        private Nullable<bool> isParsed;
        private Nullable<bool> isProofParsed;

        public SimpleEingabeverwalter(string adtPath, string taskPath, string proofPath) : base() 
        {
            this.adtPath = adtPath;
            this.taskPath = taskPath;
            this.proofPath = proofPath;
            isParsed = null;
            isProofParsed = null;
        }

        public override void verwalteEingabe()
        {
            inputContainer = new Container();
            string adtInput = File.ReadAllText(adtPath);
            string taskInput = File.ReadAllText(taskPath);
            string proofInput = File.ReadAllText(proofPath);
            try
            {
                inputContainer.ADT = ADT_Antlr.parse(adtInput);
            }
            catch (Exception e)
            {
                Console.WriteLine("Failed to parse the ADT!");
                Console.WriteLine(e.Message);
                isParsed = false;
                return;
            }
            try
            {
                Task_Antlr.parse(taskInput,inputContainer);
            }
            catch (Exception e)
            {
                Console.WriteLine("Failed to parse the Task!");
                Console.WriteLine(e.Message);
                isParsed = false;
                return;
            }
            try
            {
                Proof_Antlr.parse(proofInput,inputContainer);                
            }
            catch (Exception e)
            {
                Console.WriteLine("Failed to parse the Proof!");
                Console.WriteLine(e.Message);
                isParsed = false;
                isProofParsed = false;
                return;
            }
            isParsed = true;
            isProofParsed = true;
        }

        /// <summary>
        /// Returns null if Parsing didn't started yet. Returns false if Parsing failed. Returns true if Parsing was successful.
        /// </summary>
        /// <returns></returns>
        public Nullable<bool> IsParsed() => isParsed;
        /// <summary>
        /// Like IsParsed(), but only for Proof.
        /// </summary>
        /// <returns></returns>
        public Nullable<bool> IsProofParsed() => isProofParsed;
    }
}
