﻿using Eingabeverwaltung;
using Bewertung;
using Ausgabeverwaltung;
using System;
using System.Collections.Generic;
using System.Collections.Immutable;
using System.IO;

class ProgramForHuman
{
    private static Eingabeverwalter eingabeverwalter = new Eingabeverwalter();
    private static ImmutableDictionary<string, Action> ProgrammManager;
    private static IBewerter Bewerter;

    static void Main(string[] args)
    {
        if (args.Length == 0) eingabeverwalter.verwalteEingabe();
        else if (args.Length == 1 && args[0] == "debug") while (true) manageProgramm();
        else if (args.Length == 3) eingabeverwalter.verwalteEingabe(args[0], args[1], args[2]);
        else
        {
            Console.WriteLine("Unknown Number of Arguments " + args.Length + ". Please write 3 or no Arguments!");
            exit();
        }
        ueberpruefe();
        bewerte();
        ausgabe();
    }

    /// <summary>
    /// Can be used for Debuging.
    /// </summary>
    private static void manageProgramm()
    {
        Console.WriteLine("Was nun? ");
        string input = Console.ReadLine();
        ProgrammManager = new Dictionary<string, Action>
        {
            ["eingabe"] = eingabeverwalter.verwalteEingabe,
            ["ueberpruefe"] = ueberpruefe,
            ["bewerte"] = bewerte,
            ["ausgabe"] = ausgabe,
            ["printErrors"] = printErrors,
            ["test"] = test,
            ["exit"] = exit,
            ["?"]    = giveHelp,
        }.ToImmutableDictionary();
        try
        {
            ProgrammManager[input]();
        }
        catch (KeyNotFoundException) { Console.WriteLine("Command not found. Type ? for help."); manageProgramm(); }
    }
    private static void ueberpruefe() => Ueberpruefung.Ueberpruefer.ueberpruefe(eingabeverwalter.GetContainer());
    private static void bewerte()
    {
        try { Bewerter = new Bewerter(eingabeverwalter.GetContainer(), File.ReadAllText(".config")); }
        catch (FileNotFoundException) 
        { 
            Console.WriteLine("Info: It was no config loaded for Bewertung."); 
            Bewerter = new Bewerter(eingabeverwalter.GetContainer()); 
        }
        Bewerter.Bewerte();
    }
    private static void ausgabe() => new Ausgabeverwalter().Ausgabe(eingabeverwalter.GetContainer(), Bewerter.GetBewertung(),Bewerter.GetRound());
    private static void printErrors() => Console.WriteLine(new Ausgabeverwalter().printAllErrors(eingabeverwalter.GetContainer()));
    private static void test() => new Testverwaltung.Testverwalter(eingabeverwalter.GetContainer()).verwalteTests();
    private static void exit() => Environment.Exit(0);
    private static void giveHelp() 
    {
        foreach(string key in ProgrammManager.Keys)
            Console.WriteLine(key);
    }
}
